      subroutine fcsepatom(nz,n,h,mxo,occ,norbit,qcore,qvale,gauss,vlc, &
       vlocfac,hatpot,                                                  &
       nlst,llst,nmx,r,twfn,wfn,itype,coreden,vden1,tvden1,tvden,dum,   &
       cden,rv1,rtv1,rtv,lmax,nbase,nos,nop,nod,nof,nog,otp,otphi,ophi, &
       odphi,mxb,mxnl,tij,qij,hij,am,bm,mxwk,wk,eig,emin,               &
       mxloop,owfn,iiirc,denout,nmx3,iscr1,iscr2,small)
!  frozen core and separable approximation version of dfatom
!  program to calculate the self-consistent density functional
!    atom ground state for atom with atomic number nz
!    lmax = maximum value of l for basis states
!  iteratively solve paw hamiltonian equations to find self-consistent
!   smooth wavefunctions twfn
!    nz = nuclear charge
!    n = number of mesh points
!    h = step size between mesh points ( in bohr)
!    mxo = maximum number of orbitals
!    norbit = actual number of valence orbital
!    vlocfac = prefactor for local potential = vlocfac*exp(-(r/pgauss)^kpow)
!    llst(mxo) = l for each valence orbital
!    nlst(mxo) = principal quantum number for each valence orbital
!    occ(mxo) = occupation of each valence orbital
!    twfn =  smooth wavefunction
!    owfn =  output smooth wavefunction -- copied to twfn upon convergence
!    wfn  =  corresponding all-electron wavefunction
!    itype = exchange-correlation type index
!    coreden = core density -- input -- total core charge = qcore
!    vden1 = one center all-electron valence density  -- calculated
!    tvden = smooth valence density  -- calculated
!    tvden1 = one center smooth valence density  -- calculated
!    cden  = gaussian density corresponding to parameter gauss -- n^
!    rv1 = r * vpot (one center all-electron valence potential)
!    rtv1 = r * tvpot (one center smooth valence potential)
!    rtv = r * tvpot (smooth valence potential)
!  tij, qij are kinetic energy and overlap matrix elements (input)
!  hij (hamiltonia)n must be recalculated each iteration
!  otp are projector functions
!
        Use GlobalConstants
        Use Anderson_RealMix
      implicit real*8 (a-h,o-z)
      parameter (small0=1.d-15,rimix=0.5)
      parameter (niter=1000)
      dimension occ(mxo),twfn(nmx,mxo),vden1(nmx),tvden1(nmx),tvden(nmx)
      dimension cden(nmx3),denout(nmx3),wfn(nmx,mxo),owfn(nmx,mxo)
      dimension r(nmx),coreden(nmx),dum(nmx),wk(nmx,mxwk),vlc(nmx)
      dimension otp(nmx,mxo),otphi(nmx,mxo),ophi(nmx,mxo)
      dimension odphi(nmx,mxo),hatpot(nmx)
      dimension qij(mxb,mxb,mxnl),tij(mxb,mxb,mxnl),am(mxb,mxb),bm(mxb)
      dimension hij(mxb,mxb,mxnl)
      dimension rv1(nmx),rtv1(nmx),rtv(nmx)
      dimension eig(mxo),emin(mxo)
      dimension nlst(mxo),llst(mxo)
      type (Anderson_Context), pointer,save :: PSrho

      if (mxwk.lt.13) then
       write(6,*) 'error in fcsepatom - mxwk must be gt 13',mxwk
       stop
      endif
       if (norbit.gt.mxo) then
        write(6,*) 'error in fcsepatom -- norbit,mxo=',norbit,mxo
        stop
        endif
      !pi=4*datan(1.d0)
      rmix=rimix
!      rewind iscr1
!
!
!  calculate qvale == electrons
!
       electrons=0.d0
       do io=1,norbit
       write(6,620) nlst(io),llst(io),occ(io)
  620 format(i2,1x,i2,4x,1pe15.7)
       electrons=electrons+occ(io)
       enddo
      qion=nz-electrons-qcore
      write(6,*)
      write(6,*) 'nuclear charge    = ' , nz
      write(6,*) 'core charge    = ' , qcore
      write(6,*) 'valence electronic charge = ', electrons
      write(6,*) 'net charge        = ', qion
      qvale=electrons
!
       loop=-1
! convergence 10x larger than for original atomic calculations
       cnvrg=10*small*(nz**2)
       write(6,*) 'Density convergence parameter set to ',cnvrg
!
!    start iteration loop
!
!
!  calculate density
!
      call formdensity(n,h,vden1,tvden1,tvden,norbit,llst,mxo,nmx,      &
        twfn,occ,nbase,nos,nop,nod,nof,nog,otp,otphi,ophi,wk,wk(1,2),   &
        qvale,mxnl,mxb,qij,iiirc)
!
 2001 continue
      loop=loop+1
!
!  calculate  potential * r == rv for given density functions
!
      call formpotential(n,h,r,coreden,vden1,tvden1,tvden,vlocfac,vlc, &
        hatpot,nz,gauss,zeff,cden,dum,qcore,qvale,rv1,rtv1,rtv,tv0,tv0p, &
        nmx,mxwk,wk,itype,vr0,mxb,iiirc)
!
!   calculate hij matrix elements
!
       iii=0
       do l=0,lmax
        if (l.eq.0) many=nos
        if (l.eq.1) many=nop
        if (l.eq.2) many=nod
        if (l.eq.3) many=nof
        if (l.eq.4) many=nog
         do ib=1,many
         do ic=1,many
         call dvij(iiirc,h,l,r,ophi(1,iii+ic),ophi(1,iii+ib),rv1,       &
              otphi(1,iii+ic),otphi(1,iii+ib),rtv1,wk,wk(1,2),          &
              wk(1,3),sum)
!
           write(6,*) 'fcsep -- t,v,h',tij(ic,ib,l+1),sum,hij(ic,ib,l+1)
           hij(ic,ib,l+1)=sum+tij(ic,ib,l+1)
           enddo
           enddo
        iii=iii+many
        enddo
!
!  solve for bound states of separable Schroedinger equation
!
      icount=0
      qion=nz-electrons-qcore
      jierr=0
      it=0
      deltasum=0
!  loop through all of the states
      do io=1,norbit
      l=llst(io)
      np=nlst(io)
      energy=eig(io)
      if (io.gt.1) then
       if (l.eq.llst(io-1)) emin(io)=eig(io-1)+0.01d0
      endif
      emax=0.d0
      noz=0
!
       istart=1
       if (l.gt.0) then
       do il=0,l-1
       if (il.eq.0) many=nos
       if (il.eq.1) many=nop
       if (il.eq.2) many=nod
       if (il.eq.3) many=nof
       if (il.eq.4) many=nog
       istart=istart+many
       enddo
       endif
       if (l.eq.0) many=nos
       if (l.eq.1) many=nop
       if (l.eq.2) many=nod
       if (l.eq.3) many=nof
       if (l.eq.4) many=nog
!
      if (many+6.gt.mxwk) then
       write(6,*) 'error in boundsep --increase mxwk gt',many+6
       stop
       endif
!
         owfn(1:n,io)=twfn(1:n,io)  !call dcopy (n,twfn(1,io),1,owfn(1,io),1)
!
!  estimate energy by taking expectation value
!
      energy=expectation(n,h,l,r,twfn(1,io),rtv,nmx,many,               &
         otp(1,istart),bm,mxb,qij(1,1,l+1),hij(1,1,l+1),                &
         wk,wk(1,2),wk(1,3))
      write(6,*) 'starting boundsep with energy',energy
!
      call boundsep(n,iiirc,energy,wk(1,many+5),l,rtv,h,tv0,tv0p,       &
        otp(1,istart),nmx,hij(1,1,l+1),qij(1,1,l+1),mxb,many,am,bm,     &
        wk,wk(1,many+1),wk(1,many+2),wk(1,many+3),wk(1,many+4),         &
        emin(io),emax,niter,ierr,odphi(1,istart),wk(1,many+6),nz)
!
      check=expectation(n,h,l,r,wk(1,many+5),rtv,nmx,many,              &
         otp(1,istart),bm,mxb,qij(1,1,l+1),hij(1,1,l+1),                &
         wk,wk(1,2),wk(1,3))
      write(6,*) 'ending boundsep with ierr, energy',ierr, check
        If (ierr.ne.0) stop
!     do i=1,n
!     write(89,680) r(i),twfn(i,io),wk(i,many+5)
! 680 format(1p3e15.7)
!     enddo
!
       if (ierr.lt.9) then
         eig(io)=energy
         owfn(1:n,io)=wk(1:n,many+5) !call dcopy (n,wk(1,many+5),1,owfn(1,io),1)
         twfn(1:n,io)=wk(1:n,many+5) !call dcopy (n,wk(1,many+5),1,twfn(1,io),1)
         wfn(1:n,io)=wk(1:n,many+6) !call dcopy (n,wk(1,many+6),1,wfn(1,io),1)
        else
        jierr=jierr+1
       endif
!
      enddo
!
!  summarize results
!
      nmix=n+2*iiirc
      cden(1:nmix)=0.d0   !call dcopy(nmix,0.d0,0,cden,1)
      cden(1:iiirc)=vden1(1:iiirc) ! call dcopy(iiirc,vden1,1,cden,1)
      cden(iiirc+1:2*iiirc)=tvden1(1:iiirc)
                   ! call dcopy(iiirc,tvden1,1,cden(iiirc+1),1)
      cden(2*iiirc+1:2*iiirc+n)=tvden(1:n)
                   ! call dcopy(n,tvden,1,cden(2*iiirc+1),1)
      write(6,*) '  results for loop = ',loop
      write(6,*) ' n  l     occupancy       energy'
      do io=1,norbit
      write(6,630) nlst(io),llst(io),occ(io),eig(io)
  630 format(i2,1x,i2,4x,1p2e15.7)
      enddo
!
      denout(1:n)=0.d0  !call dcopy(nmix,0.d0,0,denout,1)
      call formdensity(n,h,vden1,tvden1,tvden,norbit,llst,mxo,nmx,      &
        twfn,occ,nbase,nos,nop,nod,nof,nog,otp,otphi,ophi,wk,wk(1,2),   &
        qvale,mxnl,mxb,qij,iiirc)
       denout(1:iiirc)=vden1(1:iiirc) !call dcopy(iiirc,vden1,1,denout,1)
       denout(iiirc+1:2*iiirc)=tvden1(1:iiirc)
             ! call dcopy(iiirc,tvden1,1,denout(iiirc+1),1)
       denout(2*iiirc+1:2*iiirc+n)=tvden(1:n) 
             ! call dcopy(n,tvden,1,denout(2*iiirc+1),1)
!
!  call rhoiter
!
!      write(iscr1) (cden(i),i=1,nmix)
!      write(iscr1) (denout(i),i=1,nmix)

      denout(1:nmix)=denout(1:nmix)-cden(1:nmix)
       deltasum=0
          do i=1,nmix
            deltasum=deltasum+ABS(denout(i))
          enddo
      write(6,*) 'fcsepatom density iter',loop,deltasum
      if (loop.eq.0) call InitAnderson(PSrho,6,5,nmix,rmix,1.d5)
      if (loop.eq.0) cnvrg=PSrho%Machaccur*1.d7*(nz*nz)
      call Anderson_Mix(PSrho,cden(1:nmix),denout(1:nmix))

      !call rhoiter(loop,nmix,cden,denout,rmix,wk,wk(1,3),wk(1,5),       &
      ! wk(1,7),wk(1,9),wk(1,11),iscr1,iscr2,mxloop,deltasum)
!  correct and recale density
       do i=1,nmix
       if (cden(i).lt.0.d0) cden(i)=0.d0
       enddo
       dum(1:n)=cden(2*iiirc+1:2*iiirc+n)
                ! call dcopy(n,cden(2*iiirc+1),1,dum,1)
       do i=1,iiirc
       dum(i)=dum(i)+cden(i)-cden(iiirc+i)
       enddo
      qcal=overint(n,h,dum)
      write(6,*) 'qcal electrons = ',qcal, electrons
!  rescale density
      rescale=1.d0
      if (qcal.gt.1.d-5) rescale=electrons/qcal
      cden(1:nmix)=cden(1:nmix)*rescale  ! call dscal(nmix,rescale,cden,1)
       vden1(1:iiirc)=cden(1:iiirc) !call dcopy(iiirc,cden,1,vden1,1)
       tvden1(1:iiirc)=cden(iiirc+1:2*iiirc)
                  ! call dcopy(iiirc,cden(iiirc+1),1,tvden1,1)
       tvden(1:n)=cden(2*iiirc+1:2*iiirc+n)  
                  ! call dcopy(n,cden(2*iiirc+1),1,tvden,1)
!
!
      write(6,*) '  results for loop ,delta = ',loop,deltasum
      write(6,*) ' n  l     occupancy       energy'
!     eone=0.d0
      do io=1,norbit
      write(6,630) nlst(io),llst(io),occ(io),eig(io)
!     eone=eone+occ(io)*eig(io)
      enddo
!     write(6,*)
!     write(6,*) ' Total energies'
!     write(6,*) '    One-electron contribution:  ',eone
!     write(6,*) '    Coulomb contribution     :  ',ecoul
!     write(6,*) '    Exch-correl contribution :  ',etxc
!     etot=eone-ecoul+etxc
!     write(6,*) '    Total                    :  ',etot
!
      if (deltasum.gt.cnvrg.or.loop.lt.4) go to 2001
       if (deltasum.gt.cnvrg.and.loop.gt.mxloop) then
       write(6,*)'calculation terminating without convergence'
       write(6,*) 'deltasum, cnvrg =', deltasum,cnvrg
       write(6,*) 'loop,mxloop = ',loop,mxloop
       stop
       endif
!
!  converged result
!
      write(6,*) '  calculation converged in',loop,' iterations'
      write(6,*) '     for nz = ',nz
      write(6,*) '    delta(density)  = ', deltasum
      write(6,*) '  results for loop = ',loop
      write(6,*) ' n  l     occupancy       energy'
!     eone=0.d0
      do io=1,norbit
      write(6,630) nlst(io),llst(io),occ(io),eig(io)
!     eone=eone+occ(io)*eig(io)
      enddo
!
!     write(6,*)
!     write(6,*) ' Total energies'
!     write(6,*) '    One-electron contribution:  ',eone
!     write(6,*) '    Coulomb contribution     :  ',ecoul
!     write(6,*) '    Exch-correl contribution :  ',etxc
!     etot=eone-ecoul+etxc
!     write(6,*) '    Total                    :  ',etot
      Call FreeAnderson(PSrho)
      return
       end
